@@ -112,6 +112,15 @@ def migration_0005(repo, init=False, **kwargs):
 
 
 def migration_0006(repo, init=False, **kwargs):
+    """ Fix a glitch preventing folders from being completely portable.
+
+    Early versions of Jirafs would write an absolute path to the ignore
+    file to the local git configuration, but that's not very desirable
+    because if you move the folder, the @stash_local_changes decorator
+    would then wipe out the git repository itself (among other things)
+    after stashing.  Whoops; that's embarrassing.
+
+    """
     if init:
         set_repo_version(repo, 6)
         return
@@ -127,3 +136,13 @@ def migration_0006(repo, init=False, **kwargs):
     )
 
     set_repo_version(repo, 6)
+
+
+def migration_0007(repo, init=False, **kwargs):
+    """ Create the plugin metadata directory."""
+    os.mkdir(
+        repo.get_metadata_path(
+            'plugin_meta',
+        )
+    )
+    set_repo_version(repo, 7)
