@@ -17,8 +17,9 @@ from . import constants
 from . import exceptions
 from . import migrations
 from . import utils
-from .decorators import stash_local_changes
+from .decorators import run_plugins, stash_local_changes
 from .jirafieldmanager import JiraFieldManager
+from .plugin import PluginValidationError
 
 
 logger = logging.getLogger(__name__)
@@ -29,7 +30,9 @@ class TicketFolder(object):
         self.path = os.path.realpath(
             os.path.expanduser(path)
         )
+        self.issue_url = self.get_ticket_url()
         self.get_jira = jira
+        self.plugins = self.load_plugins()
 
         if not os.path.isdir(self.metadata_dir):
             raise exceptions.NotTicketFolderException(
@@ -38,7 +41,6 @@ class TicketFolder(object):
                 )
             )
 
-        self.issue_url = self.get_ticket_url()
         if migrate:
             self.run_migrations()
 
@@ -48,6 +50,81 @@ class TicketFolder(object):
             with open(comment_path, 'w') as out:
                 out.write('')
 
+    def execute_plugin_method_series(
+        self, name, args=None, kwargs=None, single_response=False
+    ):
+        if args is None:
+            args = []
+            use_kwargs = True
+        if kwargs is None:
+            kwargs = {}
+            use_kwargs = False
+
+        if use_kwargs and single_response:
+            raise RuntimeError(
+                "When executing plugins in series using `single` response "
+                "mode, you must specify only args."
+            )
+        elif args and kwargs:
+            raise RuntimeError(
+                "Plugins can be ran in series using either args or "
+                "kwargs, not both."
+            )
+
+        for plugin in self.plugins:
+            if not hasattr(plugin, name):
+                continue
+            method = getattr(plugin, name)
+            plugin_result = method(*args, **kwargs)
+            if plugin_result is not None:
+                if use_kwargs:
+                    kwargs = plugin_result
+                elif single_response:
+                    args = (plugin_result, )
+                else:
+                    args = plugin_result
+
+        if use_kwargs:
+            return kwargs
+        elif single_response:
+            return args[0]
+        return args
+
+    def load_plugins(self):
+        config = self.get_config()
+        plugins = []
+
+        if not config.has_section(constants.CONFIG_PLUGINS):
+            return plugins
+
+        installed_plugins = utils.get_installed_plugins()
+
+        for name, status in config.items(constants.CONFIG_PLUGINS):
+            if not utils.convert_to_boolean(status):
+                # This plugin is not turned on.
+                continue
+            if name not in installed_plugins:
+                # This plugin is not installed.
+                self.log(
+                    "Plugin '%s' is not available.",
+                    (name, ),
+                )
+                continue
+
+            plugin = installed_plugins[name](self, name)
+
+            try:
+                plugin.validate()
+            except PluginValidationError as e:
+                self.log(
+                    "Plugin '%s' did not pass validation; not loading: %s.",
+                    (name, e,)
+                )
+
+            plugins.append(plugin)
+
+        return plugins
+
     def get_config(self):
         local_config_file = self.get_metadata_path('config')
         additional_configs = []
@@ -192,11 +269,22 @@ class TicketFolder(object):
         )
         try:
             with open(remote_files, 'r') as _in:
-                return json.loads(_in.read())
+                data = json.loads(_in.read())
         except IOError:
-            return {}
+            data = {}
+
+        return self.execute_plugin_method_series(
+            'alter_get_remote_file_metadata',
+            args=(data, ),
+            single_response=True,
+        )
 
     def set_remote_file_metadata(self, data, shadow=True):
+        data = self.execute_plugin_method_series(
+            'alter_set_remote_file_metadata',
+            args=(data, ),
+            single_response=True,
+        )
         remote_files = self.get_path(
             '.jirafs/remote_files.json',
             shadow=shadow
@@ -413,7 +501,9 @@ class TicketFolder(object):
     def get_ready_changes(self):
         changed_files = self.filter_ignored_files(
             self.run_git_command(
-                'diff', '--name-only', self.git_merge_base,
+                'diff',
+                '--name-only',
+                '%s..master' % self.git_merge_base,
             ).split('\n')
         )
 
@@ -454,7 +544,11 @@ class TicketFolder(object):
             if changed:
                 assets.append(attachment.filename)
 
-        return assets
+        return self.execute_plugin_method_series(
+            name='alter_remotely_changed',
+            args=(assets, ),
+            single_response=True,
+        )
 
     def filter_ignored_files(self, files, which=constants.IGNORE_FILE):
         ignore_globs = self.get_ignore_globs(which)
@@ -479,7 +573,11 @@ class TicketFolder(object):
                 continue
             assets.append(fileish)
 
-        return assets
+        return self.execute_plugin_method_series(
+            name='alter_filter_ignored_files',
+            args=(assets, ),
+            single_response=True,
+        )
 
     def get_fields(self, revision=None, path=None):
         kwargs = {}
@@ -518,25 +616,38 @@ class TicketFolder(object):
                     self.get_local_path(constants.TICKET_NEW_COMMENT), 'r+'
                 ) as c:
                     c.truncate()
-
-            return contents
         except IOError:
-            return ''
+            contents = ''
 
+        return self.execute_plugin_method_series(
+            name='alter_new_comment',
+            args=(contents, ),
+            single_response=True,
+        )
+
+    @run_plugins(pre='pre_fetch', post='post_fetch')
     def fetch(self):
         file_meta = self.get_remote_file_metadata(shadow=True)
+        original_hash = self.run_git_command('rev-parse', 'jira')
 
         for filename in self.get_remotely_changed():
             for attachment in self.issue.fields.attachment:
                 if attachment.filename == filename:
-                    shadow_filename = self.get_shadow_path(filename)
-                    with open(shadow_filename, 'wb') as download:
-                        self.log(
-                            'Download file "%s"',
-                            (attachment.filename, ),
-                        )
+                    self.log(
+                        'Download file "%s"',
+                        (attachment.filename, ),
+                    )
+                    content = six.BytesIO(attachment.get())
+                    filename, content = self.execute_plugin_method_series(
+                        'alter_file_download',
+                        args=((filename, content, ),),
+                        single_response=True,
+                    )
+                    save_path = self.get_shadow_path(filename)
+                    with open(save_path, 'wb') as save_file:
+                        content.seek(0)
+                        save_file.write(content.read())
                         file_meta[filename] = attachment.created
-                        download.write(attachment.get())
 
         self.set_remote_file_metadata(file_meta, shadow=True)
 
@@ -611,10 +722,23 @@ class TicketFolder(object):
             failure_ok=True, shadow=True
         )
         self.run_git_command('push', 'origin', 'jira', shadow=True)
+        final_hash = self.run_git_command('rev-parse', 'jira')
+        return utils.PostStatusResponse(
+            original_hash == final_hash,
+            final_hash
+        )
 
     @stash_local_changes
+    @run_plugins(pre='pre_merge', post='post_merge')
     def merge(self):
+        original_merge_base = self.git_merge_base
         self.run_git_command('merge', 'jira')
+        final_merge_base = self.git_merge_base
+
+        return utils.PostStatusResponse(
+            original_merge_base == final_merge_base,
+            final_merge_base
+        )
 
     def pull(self):
         self.fetch()
@@ -644,8 +768,10 @@ class TicketFolder(object):
         return True
 
     @stash_local_changes
+    @run_plugins(pre='pre_push', post='post_push')
     def push(self):
         status = self.status()
+        original_hash = self.run_git_command('rev-parse', 'jira')
 
         if not self.is_up_to_date():
             raise exceptions.LocalCopyOutOfDate()
@@ -660,6 +786,11 @@ class TicketFolder(object):
                     binary=True
                 )
             )
+            filename, upload = self.execute_plugin_method_series(
+                'alter_file_upload',
+                args=((filename, upload, ), ),
+                single_response=True,
+            )
             self.log(
                 'Uploading file "%s"',
                 (filename, ),
@@ -668,6 +799,7 @@ class TicketFolder(object):
             for attachment in self.issue.fields.attachment:
                 if attachment.filename == filename:
                     attachment.delete()
+            upload.seek(0)
             attachment = self.jira.add_attachment(
                 self.ticket_number,
                 upload,
@@ -715,7 +847,13 @@ class TicketFolder(object):
             failure_ok=True, shadow=True
         )
         self.run_git_command('push', 'origin', 'jira', shadow=True)
+        final_hash = self.run_git_command('rev-parse', 'jira')
+        return utils.PostStatusResponse(
+            original_hash == final_hash,
+            final_hash
+        )
 
+    @run_plugins(pre='pre_status', post='post_status')
     def status(self):
         status = {
             'uncommitted': self.get_uncommitted_changes(),
