@@ -1,16 +1,19 @@
+import collections
 import getpass
 import os
+import pkg_resources
 
 from jira.client import JIRA
 from six.moves import configparser, input
 
 from . import constants
+from .plugin import Plugin
 
 
 def convert_to_boolean(string):
-    if string.upper().strip() in ['Y', 'YES']:
+    if string.upper().strip() in ['Y', 'YES', 'ON', 'ENABLED']:
         return True
-    elif string.upper().strip() in ['N', 'NO']:
+    elif string.upper().strip() in ['N', 'NO', 'OFF', 'DISABLED']:
         return False
     return None
 
@@ -40,6 +43,22 @@ def get_user_input(message, options=None, boolean=False, password=False):
     return value
 
 
+def get_installed_plugins():
+    possible_plugins = {}
+    for entry_point in (
+        pkg_resources.iter_entry_points(group='jirafs_plugins')
+    ):
+        try:
+            loaded_class = entry_point.load()
+        except ImportError:
+            continue
+        if not issubclass(loaded_class, Plugin):
+            continue
+        possible_plugins[entry_point.name] = loaded_class
+
+    return possible_plugins
+
+
 def get_config(additional_configs=None, include_global=True):
     filenames = []
     if include_global:
@@ -133,3 +152,9 @@ def get_jira(domain=None, config=None):
 
 def lazy_get_jira():
     return lambda domain, config=None: get_jira(domain, config)
+
+
+PostStatusResponse = collections.namedtuple(
+    'PostStatusResponse',
+    ['new', 'hash']
+)
