@@ -2,52 +2,14 @@ import json
 import os
 import shutil
 import tempfile
-from unittest import TestCase
 
 import mock
 from mock import patch
-from jira.resources import Issue
 
 from jirafs import exceptions
 from jirafs.ticketfolder import TicketFolder
 
-
-class BaseTestCase(TestCase):
-    def get_asset_path(self, filename):
-        return os.path.join(
-            os.path.dirname(__file__),
-            'assets',
-            filename
-        )
-
-    def get_asset_contents(self, filename, mode='r'):
-        path = self.get_asset_path(filename)
-
-        with open(path, mode) as r:
-            return r.read()
-
-    def rehydrate_issue(self, filename):
-        stored = json.loads(self.get_asset_contents(filename))
-        return Issue(
-            stored['options'],
-            None,
-            stored['raw'],
-        )
-
-    def get_empty_status(self):
-        return {
-            'ready': {
-                'files': [],
-                'fields': {},
-                'new_comment': '',
-            },
-            'uncommitted': {
-                'files': [],
-                'fields': {},
-                'new_comment': '',
-            },
-            'up_to_date': True,
-        }
+from .base import BaseTestCase
 
 
 class TestTicketFolder(BaseTestCase):
